#ifndef TEST_DATASET_H_
#define TEST_DATASET_H_

#define nb_samples {{nb_tests}}
#define nn_input_size {{nb_inputs}}
#define nn_output_size {{nb_outputs}}

extern {{data_type}} nn_test_inputs[nb_samples][nn_input_size];

#endif