
// Normalization cst
extern float input_min[{{input_size}}];
extern float input_max[{{input_size}}];
extern float input_mean[{{input_size}}];
extern float input_range[{{input_size}}];
extern float output_mean;
extern float output_range;
