
// Normalization cst
extern float input_min[{{input_size}}] = {{input_min}};
extern float input_max[{{input_size}}] = {{input_max}};
extern float input_mean[{{input_size}}] = {{input_mean}};
extern float input_range[{{input_size}}] = {{input_range}};
extern float output_mean = {{output_mean}};
extern float output_range = {{output_range}};